@@ -8,6 +8,7 @@ Rcpp::NumericVector binner_cont(
       Rcpp::NumericVector param,
       int k=10,
       int which=1,
-      Rcpp::NumericVector Range=Rcpp::NumericVector::create(-99, -99));
+      Rcpp::NumericVector Range=Rcpp::NumericVector::create(-99, -99),
+      double minexpcount=2.0);
 
 #endif
