@@ -6,16 +6,17 @@ using namespace Rcpp;
 //' @param  x An integer vector of counts.
 //' @param  p cumulative distribution function values at vals
 //' @param  k number of desired bins
+//' @param  minexpcount =2 minimal expected bin count required
 //' @keywords internal
 //' @return An integer vector of indices
 // [[Rcpp::export]]
 Rcpp::IntegerVector binner_disc(
     Rcpp::IntegerVector x, 
     Rcpp::NumericVector p, 
-    int k=10) {
+    int k=10,
+    double minexpcount=2) {
   int n=sum(x), i;
   double tmp;
-  const int minE=2;
   NumericVector E(x.size());  
   IntegerVector out(x.size());
   for(i=0;i<x.size();++i) out[i]=i;
@@ -25,7 +26,7 @@ Rcpp::IntegerVector binner_disc(
   
   if(k>x.size()) k = x.size();
   
-  while ( (E.size()>k) || (min(E)<minE)) {
+  while ( (E.size()>k) || (min(E)<minexpcount)) {
       int whichmin=0;
       tmp=E[0];
       for(i=1;i<E.size();++i) {
