@@ -4,11 +4,11 @@
 #include <Rcpp.h>
 Rcpp::NumericMatrix gof_disc(Rcpp::IntegerVector x, 
                        Rcpp::Function pnull, 
-                       Rcpp::NumericVector vals,                        
                        Rcpp::Function rnull, 
+                       Rcpp::NumericVector vals,                        
                        Rcpp::Function phat,
+                       Rcpp::Function TS,
                        double rate=0, 
-                       int B=5000,
-             Rcpp::CharacterVector doMethod=Rcpp::CharacterVector::create("KS", "Kuiper", "AD", "CvM", "W", "ZA", "ZK", "ZC", "Wassp1"));
+                       int B=5000);
 
 #endif
